/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * ToOutChan.{cc,hh} -- Send packets to a OSKit netio.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "tooutchan.hh"
#include "error.hh"
#include "packet.hh"
#include "confparse.hh"
#include "glue.hh"
#include <unistd.h>

extern "C" {
#include <nodeos/impl/ani_flow.h>
#include <nodeos/impl/ani_mempool.h>
}

ToOutChan::ToOutChan()
  : Element(1, 0)
{
}

ToOutChan::~ToOutChan()
{
}

ToOutChan *
ToOutChan::clone() const
{
	return new ToOutChan;
}

int
ToOutChan::configure(const String &, ErrorHandler *)
{
	return 0;
}

int
ToOutChan::initialize(ErrorHandler *)
{
	return 0;
}

/*
 * Call back to free the packet memory when we create an external version
 * Does nothing cause the memory is freed up in the Click packet destructor.
 * This will need to change when nodeos packets are ref counted.
 */
extern "C" void
ToOutChan_packet_free(ani_pbuf_t *, void *)
{
}

void
ToOutChan::push(int, Packet *p)
{
	/*
	 * Kludgey. 
	 */
	ani_pbuf_t	*pbuf = p->nodeos_packet();

	if (pbuf == 0) {
		size_t	off, bsize;
		
		pbuf = (ani_pbuf_t *)
			ani_mempool_alloc(ani_mempool_current(), sizeof *pbuf);
		if (pbuf == 0) {
			click_chatter("ToOutChan::push: "
				      "Packet creation failed!\n");
			return;
		}

		off = p->headroom();
		bsize = off + p->length() + p->tailroom();
		ani_pbuf_init_external(pbuf, ani_flow_current(), _chan,
				       p->data() - off, bsize, off, p->length(),
				       ToOutChan_packet_free, &pbuf);

		/*
		 * The packet have a ref count > than 1, so someone else
		 * is holding it. Give it the nodeos packet pointer so
		 * we don't have to go through this again.
		 */
		p->set_nodeos_packet(pbuf);
	}
	ani_chan_send(_chan, pbuf);
	p->kill();
}

bool
ToOutChan::wants_packet_upstream() const
{
	return input_is_pull(0);
}

void
ToOutChan::run_scheduled()
{
	while (Packet *p = input(0).pull())
		push(0, p);
}

EXPORT_ELEMENT(ToOutChan)
