/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef FROMNETIO_HH
#define FROMNETIO_HH

#include "element.hh"
#include "string.hh"

extern "C" {
#include <oskit/io/skbufio.h>
}

class FromNetIO : public Element {
public:
	FromNetIO();
	~FromNetIO();
  
	const char *class_name() const		{ return "FromNetIO"; }
	const char *processing() const		{ return PUSH; }
  
	FromNetIO *clone() const;
	int configure(const String &, ErrorHandler *);
	int initialize(ErrorHandler *);
  
	/* process a packet. return 0 if not wanted after all. */
	int got_packet(struct sk_buff *skb);

	/* So we can run from the event loop */
	void run_scheduled();

private:
	int	_ifnum;
	void	*_priv;
	int	_recvs;

	/*
	 * We implement a simple safe queue that stores packets until
	 * the driver loop can pick them up.
	 */
	int	_qhead;
	int	_qtail;
	int	_max;
	int	_qdrops;
	Packet **_q;

	int	next_i(int i) const		{ return (i!=_max ? i+1 : 0); }
	bool	qempty() const			{ return _qhead == _qtail; }
	void	enq(Packet *);
	Packet *deq();
};

#endif /* FROMNETIO_HH */
