#ifndef ALIGN_HH
#define ALIGN_HH
#include "element.hh"

/*
 * =c
 * Align(MODULUS, OFFSET)
 * =d
 *
 * Aligns packet data. Each input packet is aligned so that its first byte is
 * OFFSET bytes off from a MODULUS-byte boundary. This may involve a packet
 * copy.
 *
 * MODULUS must be 2, 4, or 8.
 * =n
 * The click-align tool will insert this element automatically wherever it is
 * required.
 * =e
 * = ... -> Align(4, 0) -> ...
 * =a AlignmentInfo
 * =a click-align(1) */

class Align : public Element {

  int _offset;
  int _mask;
  
 public:
  
  Align();
  
  const char *class_name() const		{ return "Align"; }
  const char *processing() const		{ return AGNOSTIC; }
  
  Align *clone() const				{ return new Align; }
  int configure(const String &, ErrorHandler *);

  Packet *smaction(Packet *);
  void push(int, Packet *);
  Packet *pull(int);
  
};

#endif
