/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Trivial little test program demonstrating the use of a Click COM object
 * within a single threaded OSKit kernel.
 */

#include <stdio.h>
#include <oskit/dev/dev.h>
#include <oskit/startup.h>
#include <oskit/clientos.h>
#include "click_com.h"

#define WAITFOR		90 /* Run for this long (seconds) */

static oskit_clicker_t	*clicker;
static volatile int	stopped;
static int		waitfor = WAITFOR;


#ifdef MULTITHREADED
#define oskit_clientos_init oskit_clientos_init_pthreads
#define start_fs_bmod start_fs_bmod_pthreads
#else
#define start_pthreads()
#endif

int
main(int argc, char **argv)
{
	void		handler(void);
	char		*config_filename = "ping.click";
	char		*opt, *getenv();
	
	oskit_clientos_init();
	start_clock();
	start_pthreads();	/* Might be defined away (see above) */
#ifdef OSKIT_UNIX
	start_fs_native(".");
#else
#ifdef  GPROF
	start_fs("sd0", "a");
#else
	start_fs_bmod();
#endif
#endif
	start_net_devices();

	if ((opt = getenv("CONFIG_FILE")) != NULL)
		config_filename = opt;

	if ((opt = getenv("WAITFOR")) != NULL)
		waitfor = atoi(opt);

	/*
	 * Use a simple timer to stop after 30 seconds.
	 */
	osenv_timer_init();
	osenv_timer_register(handler, 100);

	/*
	 * Okay, lets create a Click object.
	 */
	clicker = oskit_clicker_create();

	/*
	 * Give it a configuration file
	 */
	oskit_clicker_config(clicker, config_filename);

#ifdef  GPROF
	start_gprof();
#endif
	/*
	 * Lets Click.
	 */
	oskit_clicker_start(clicker);

#ifdef  MULTITHREADED
	pthread_yield();
	
	while (! oskit_clicker_stopped(clicker))
		oskit_pthread_sleep((long long) 100);
	oskit_pthread_sleep((long long) 100);
#endif

	/*
	 * Kill the clicker
	 */
	oskit_clicker_release(clicker);

	printf("exiting ...\n");
	exit(0);
}

void
handler(void)
{
	static count = 0, sec=0;
	
	if (++count >= 100) {
		count = 0;
		if (++sec > waitfor) {
			
			/*
			 * Stop the Clicker.
			 */
			if (!stopped) {
				stopped = 1;
				printf("Stopping the Clicker(s)\n");
				oskit_clicker_stop(clicker);
			}
		}
	}
}
