/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Trivial little test program demonstrating the use of a Click COM object
 * within a multi threaded OSKit kernel.
 */

#include <stdio.h>
#include <oskit/dev/dev.h>
#include <oskit/startup.h>
#include <oskit/clientos.h>
#include "click_com.h"

#define WAITFOR		30 /* Run for this long */

static oskit_clicker_t	*clicker1, *clicker2;
static int		waitfor = WAITFOR;

int
main(int argc, char **argv)
{
	void		handler(void);
	char		*config_filename1 = "cutthrough-a.click";
	char		*config_filename2 = "cutthrough-b.click";
	char		*opt, *getenv();
	
	oskit_clientos_init_pthreads();
	start_clock();
	start_pthreads();
#ifdef OSKIT_UNIX
	start_fs_native(".");
#else
#ifdef  GPROF
	start_fs("sd0", "a");
#else
	start_fs_bmod_pthreads();
#endif
#endif
	start_net_devices();

	if ((opt = getenv("CONFIG_FILE1")) != NULL)
		config_filename1 = opt;

	if ((opt = getenv("CONFIG_FILE2")) != NULL)
		config_filename2 = opt;

	if ((opt = getenv("WAITFOR")) != NULL)
		waitfor = atoi(opt);

	/*
	 * Use a simple timer to stop after a while.
	 */
	osenv_timer_init();
	osenv_timer_register(handler, 100);

	/*
	 * Okay, lets create two Click objects.
	 */
	clicker1 = oskit_clicker_create();
	clicker2 = oskit_clicker_create();

	/*
	 * Give them a configuration file
	 */
	oskit_clicker_config(clicker1, config_filename1);
	oskit_clicker_config(clicker2, config_filename2);

#ifdef  GPROF
	start_gprof();
#endif
	/*
	 * Lets Click.
	 */
	oskit_clicker_start(clicker1);
	pthread_yield();
	oskit_clicker_start(clicker2);
	pthread_yield();
	
	while (! (oskit_clicker_stopped(clicker1) &&
		  oskit_clicker_stopped(clicker2)))
		oskit_pthread_sleep((long long) 100);
	
	oskit_pthread_sleep((long long) 100);

	/*
	 * Kill the clickers
	 */
	oskit_clicker_release(clicker1);
	oskit_clicker_release(clicker2);

	printf("exiting ...\n");
	exit(0);
}

void
handler(void)
{
	static count = 0, sec=0, stopped=0;
	
	if (++count >= 100) {
		count = 0;
		if (++sec > waitfor) {
			
			/*
			 * Stop the Clicker.
			 */
			if (!stopped) {
				stopped = 1;
				printf("Stopping the Clicker(s)\n");
				oskit_clicker_stop(clicker1);
				oskit_clicker_stop(clicker2);
			}
		}
	}
}
